from edgegap_scheduling import SchedulingSingleton, Task, errors
from fastapi import FastAPI, HTTPException
from fastapi.routing import APIRouter
from pydantic import BaseModel, Field


class TaskParameters(BaseModel):
    parameters: dict | None = Field(default=None, description='The parameters to update in the tasks')


class UpdateTask(TaskParameters):
    every: str | None = Field(default=None, description='The interval to run the Task')
    delay: str | None = Field(default=None, description='Delay before starting the Task')


class SchedulingAPI:
    tags = ['Scheduling']

    def __init__(self):
        self.__instance = SchedulingSingleton.scheduler()

    def init_scheduling_api(self, app: FastAPI):
        router = APIRouter(prefix='/scheduling/tasks', tags=self.tags)

        @router.get('/', description='Get all Tasks')
        async def get_tasks() -> list[Task]:
            return await self.__instance.list()

        @router.get('/{identifier}', description='Get One Task')
        async def get_task(identifier: str) -> Task:
            try:
                return await self.__instance.get(identifier)
            except ValueError as e:
                raise HTTPException(status_code=404, detail=str(e))

        @router.post('/{identifier}', description='Start a stopped Task')
        async def start_task(identifier: str) -> Task:
            try:
                return await self.__instance.start(identifier)
            except ValueError as e:
                raise HTTPException(status_code=404, detail=str(e))

        @router.patch('/{identifier}', description='Update the Interval of a Task')
        async def update_task(identifier: str, task: UpdateTask) -> Task:
            try:
                return await self.__instance.update(
                    identifier=identifier,
                    every=task.every,
                    delay=task.delay,
                    parameters=task.parameters,
                )
            except ValueError as e:
                raise HTTPException(status_code=404, detail=str(e))

        @router.delete('/{identifier}', description='Stop a running Task')
        async def stop_task(identifier: str) -> Task:
            try:
                return await self.__instance.stop(identifier)
            except ValueError as e:
                raise HTTPException(status_code=404, detail=str(e))

        @router.post('/run/{identifier}')
        async def run_rask(identifier: str, task_parameters: TaskParameters | None) -> Task:
            try:
                params = task_parameters.parameters if task_parameters else None
                return await self.__instance.run(identifier, params)
            except ValueError as e:
                raise HTTPException(status_code=404, detail=str(e))
            except errors.ManualRunNotAllowedError as e:
                raise HTTPException(status_code=403, detail=str(e))

        app.include_router(router)



from edgegap_settings import SettingsFactory
from pydantic import BaseModel

from .._environment import EnvironmentConfiguration

env_config: EnvironmentConfiguration = SettingsFactory.from_settings(EnvironmentConfiguration)


class LoggingConfiguration(BaseModel):
    version: int = 1
    disable_existing_loggers: bool = False
    formatters: dict = {
        'default': {
            '()': 'edgegap_logging.DefaultFormatter',
            'fmt': env_config.log_format,
        },
        'access': {
            '()': 'edgegap_service.AccessFormatter',
            'fmt': env_config.log_access_format,
        },
    }
    handlers: dict = {
        'default': {
            'formatter': 'default',
            'class': 'logging.StreamHandler',
            'stream': 'ext://sys.stdout',
        },
        'access': {
            'formatter': 'access',
            'class': 'logging.StreamHandler',
            'stream': 'ext://sys.stdout',
        },
    }
    loggers: dict = {
        'root': {
            'handlers': ['default'],
            'level': env_config.log_level,
        },
        'uvicorn': {
            'handlers': ['default'],
            'level': env_config.log_level,
            'propagate': False,
        },
        'uvicorn.error': {
            'level': env_config.log_level,
        },
        'uvicorn.access': {
            'handlers': ['access'],
            'level': env_config.log_level,
            'propagate': False,
        },
        'gunicorn.error': {
            'level': env_config.log_level,
            'handlers': ['default'],
            'propagate': False,
        },
        'gunicorn.access': {
            'level': env_config.log_level,
            'handlers': ['access'],
            'propagate': False,
        },
        'elasticapm': {
            'level': 'INFO',
            'handlers': ['default'],
            'propagate': False,
        },
        'urllib3': {
            'level': 'INFO',
            'handlers': ['default'],
            'propagate': False,
        },
    }
    root: dict = {
        'level': env_config.log_level,
        'handlers': ['default'],
    }



from ._service import ServiceFactory, ServiceConfiguration
from ._templating import TemplateRenderer
from .health import checks
from .logging import AccessFormatter

__all__ = [
    'ServiceFactory',
    'ServiceConfiguration',
    'checks',
    'AccessFormatter',
    'TemplateRenderer',
]



from typing import Callable

from fastapi import Depends, FastAPI, Request, responses

from ._model import HealthResponse
from .checks import CheckInterface


class HealthCheck:
    def __init__(self, title: str, version: str, checks: list[type(CheckInterface)], depend: Callable):
        self.__title = title
        self.__version = version

        self.__checks = [
            check if isinstance(check, CheckInterface) else check()  # Instantiate if not
            for check in checks
        ]
        self.__depend = depend

    def init_health_check(self, app: FastAPI):
        def empty_depend():
            return None

        @app.get('/health', response_model=HealthResponse, tags=['Monitoring'])
        async def health(
            request: Request,
            session=Depends(self.__depend or empty_depend),
        ) -> responses.JSONResponse:
            checks = []
            kwargs = {'session': session, 'request': request}

            for check in self.__checks:
                checks.append(check.check(**kwargs))

            all_ok = all([check.ok for check in checks])
            response = HealthResponse(
                name=self.__title,
                version=self.__version,
                all_ok=all_ok,
                checks=checks,
            )

            return responses.JSONResponse(
                content=response.model_dump(),
                status_code=200 if all_ok else 500,
            )



import abc

from ._model import CheckInstanceModel, CheckModel


class CheckInterface(abc.ABC):
    __name__: str

    def __init__(self, name: str = None):
        self.__name = name if isinstance(name, str) else self.__name__

    def check(self, **kwargs) -> CheckModel:
        instance = self._check(**kwargs)

        return CheckModel(name=self.__name, **instance.model_dump())

    @abc.abstractmethod
    def _check(self, **kwargs) -> CheckInstanceModel:
        """

        @return: tuple[bool, str]
        bool: if the check is ok
        str: message
        """
